@@ -14,13 +14,17 @@
 
 import jinja2
 import yaml
+import itertools
 import re
 import os
-from typing import Any, DefaultDict, Dict, Mapping
+import pathlib
+import typing
+from typing import Any, DefaultDict, Dict, Mapping, Tuple
 from hashlib import sha256
 from collections import OrderedDict, defaultdict
-from gapic.samplegen_utils.utils import coerce_response_name, is_valid_sample_cfg
+from gapic.samplegen_utils.utils import coerce_response_name, is_valid_sample_cfg, render_format_string
 from gapic.samplegen_utils.types import DuplicateSample
+from gapic.samplegen_utils import snippet_index, snippet_metadata_pb2
 from gapic.samplegen import manifest, samplegen
 from gapic.generator import formatter
 from gapic.schema import api
@@ -49,14 +53,22 @@ class Generator:
             loader=jinja2.FileSystemLoader(searchpath=opts.templates),
             undefined=jinja2.StrictUndefined,
             extensions=["jinja2.ext.do"],
+            trim_blocks=True,
+            lstrip_blocks=True,
         )
 
         # Add filters which templates require.
         self._env.filters["rst"] = utils.rst
         self._env.filters["snake_case"] = utils.to_snake_case
+        self._env.filters["camel_case"] = utils.to_camel_case
         self._env.filters["sort_lines"] = utils.sort_lines
         self._env.filters["wrap"] = utils.wrap
         self._env.filters["coerce_response_name"] = coerce_response_name
+        self._env.filters["render_format_string"] = render_format_string
+
+        # Add tests to determine type of expressions stored in strings
+        self._env.tests["str_field_pb"] = utils.is_str_field_pb
+        self._env.tests["msg_field_pb"] = utils.is_msg_field_pb
 
         self._sample_configs = opts.sample_configs
 
@@ -80,16 +92,27 @@ class Generator:
         sample_templates, client_templates = utils.partition(
             lambda fname: os.path.basename(
                 fname) == samplegen.DEFAULT_TEMPLATE_NAME,
-            self._env.loader.list_templates(),
+            self._env.loader.list_templates(),  # type: ignore
         )
 
+        # We generate code snippets *before* the library code so snippets
+        # can be inserted into method docstrings.
+        snippet_idx = snippet_index.SnippetIndex(api_schema)
+        if sample_templates:
+            sample_output, snippet_idx = self._generate_samples_and_manifest(
+                api_schema, snippet_idx, self._env.get_template(
+                    sample_templates[0]),
+                opts=opts,
+            )
+            output_files.update(sample_output)
+
         # Iterate over each template and add the appropriate output files
         # based on that template.
         # Sample templates work differently: there's (usually) only one,
         # and instead of iterating over it/them, we iterate over samples
         # and plug those into the template.
         for template_name in client_templates:
-            # Sanity check: Skip "private" templates.
+            # Quick check: Skip "private" templates.
             filename = template_name.split("/")[-1]
             if filename.startswith("_") and filename != "__init__.py.j2":
                 continue
@@ -97,15 +120,9 @@ class Generator:
             # Append to the output files dictionary.
             output_files.update(
                 self._render_template(
-                    template_name, api_schema=api_schema, opts=opts)
+                    template_name, api_schema=api_schema, opts=opts, snippet_index=snippet_idx)
             )
 
-        output_files.update(
-            self._generate_samples_and_manifest(
-                api_schema, self._env.get_template(sample_templates[0]),
-            )
-        )
-
         # Return the CodeGeneratorResponse output.
         res = CodeGeneratorResponse(
             file=[i for i in output_files.values()])  # type: ignore
@@ -113,15 +130,16 @@ class Generator:
         return res
 
     def _generate_samples_and_manifest(
-        self, api_schema: api.API, sample_template: jinja2.Template,
-    ) -> Dict[str, CodeGeneratorResponse.File]:
+            self, api_schema: api.API, index: snippet_index.SnippetIndex, sample_template: jinja2.Template, *, opts: Options) -> Tuple[Dict, snippet_index.SnippetIndex]:
         """Generate samples and samplegen manifest for the API.
 
         Arguments:
             api_schema (api.API): The schema for the API to which the samples belong.
+            sample_template (jinja2.Template): The template to use to generate samples.
+            opts (Options): Additional generator options.
 
         Returns:
-            Dict[str, CodeGeneratorResponse.File]: A dict mapping filepath to rendered file.
+            Tuple[Dict[str, CodeGeneratorResponse.File], snippet_index.SnippetIndex] : A dict mapping filepath to rendered file.
         """
         # The two-layer data structure lets us do two things:
         # * detect duplicate samples, which is an error
@@ -129,61 +147,61 @@ class Generator:
         id_to_hash_to_spec: DefaultDict[str,
                                         Dict[str, Any]] = defaultdict(dict)
 
-        STANDALONE_TYPE = "standalone"
-        for config_fpath in self._sample_configs:
-            with open(config_fpath) as f:
-                configs = yaml.safe_load_all(f.read())
+        # Autogenerated sample specs
+        autogen_specs: typing.List[typing.Dict[str, Any]] = []
+        if opts.autogen_snippets:
+            autogen_specs = list(
+                samplegen.generate_sample_specs(api_schema, opts=opts))
 
-            spec_generator = (
-                spec
-                for cfg in configs
-                if is_valid_sample_cfg(cfg)
-                for spec in cfg.get("samples", [])
-                # If unspecified, assume a sample config describes a standalone.
-                # If sample_types are specified, standalone samples must be
-                # explicitly enabled.
-                if STANDALONE_TYPE in spec.get("sample_type", [STANDALONE_TYPE])
-            )
+        # Also process any handwritten sample specs
+        handwritten_specs = samplegen.parse_handwritten_specs(
+            self._sample_configs)
 
-            for spec in spec_generator:
-                # Every sample requires an ID, preferably provided by the
-                # samplegen config author.
-                # If no ID is provided, fall back to the region tag.
-                # If there's no region tag, generate a unique ID.
-                #
-                # Ideally the sample author should pick a descriptive, unique ID,
-                # but this may be impractical and can be error-prone.
-                spec_hash = sha256(str(spec).encode("utf8")).hexdigest()[:8]
-                sample_id = spec.get("id") or spec.get(
-                    "region_tag") or spec_hash
-                spec["id"] = sample_id
+        sample_specs = autogen_specs + list(handwritten_specs)
 
-                hash_to_spec = id_to_hash_to_spec[sample_id]
-                if spec_hash in hash_to_spec:
-                    raise DuplicateSample(
-                        f"Duplicate samplegen spec found: {spec}")
+        for spec in sample_specs:
+            # Every sample requires an ID. This may be provided
+            # by a samplegen config author.
+            # If no ID is provided, fall back to the region tag.
+            #
+            # Ideally the sample author should pick a descriptive, unique ID,
+            # but this may be impractical and can be error-prone.
+            spec_hash = sha256(str(spec).encode("utf8")).hexdigest()[:8]
+            sample_id = spec.get("id") or spec.get("region_tag") or spec_hash
+            spec["id"] = sample_id
 
-                hash_to_spec[spec_hash] = spec
+            hash_to_spec = id_to_hash_to_spec[sample_id]
 
-        out_dir = "samples"
+            if spec_hash in hash_to_spec:
+                raise DuplicateSample(
+                    f"Duplicate samplegen spec found: {spec}")
+
+            hash_to_spec[spec_hash] = spec
+
+        out_dir = "samples/generated_samples"
         fpath_to_spec_and_rendered = {}
         for hash_to_spec in id_to_hash_to_spec.values():
             for spec_hash, spec in hash_to_spec.items():
                 id_is_unique = len(hash_to_spec) == 1
-                # The ID is used to generate the file name and by sample tester
-                # to link filenames to invoked samples. It must be globally unique.
+                # The ID is used to generate the file name. It must be globally unique.
                 if not id_is_unique:
                     spec["id"] += f"_{spec_hash}"
 
-                sample = samplegen.generate_sample(
+                sample, snippet_metadata = samplegen.generate_sample(
                     spec, api_schema, sample_template,)
 
-                fpath = spec["id"] + ".py"
+                fpath = utils.to_snake_case(spec["id"]) + ".py"
                 fpath_to_spec_and_rendered[os.path.join(out_dir, fpath)] = (
                     spec,
                     sample,
                 )
 
+                snippet_metadata.file = fpath
+                snippet_metadata.title = fpath
+
+                index.add_snippet(
+                    snippet_index.Snippet(sample, snippet_metadata))
+
         output_files = {
             fname: CodeGeneratorResponse.File(
                 content=formatter.fix_whitespace(sample), name=fname
@@ -191,25 +209,18 @@ class Generator:
             for fname, (_, sample) in fpath_to_spec_and_rendered.items()
         }
 
-        # Only generate a manifest if we generated samples.
-        if output_files:
-            manifest_fname, manifest_doc = manifest.generate(
-                (
-                    (fname, spec)
-                    for fname, (spec, _) in fpath_to_spec_and_rendered.items()
-                ),
-                api_schema,
-            )
+        if index.metadata_index.snippets:
+            # NOTE(busunkim): Not all fields are yet populated in the snippet metadata.
+            # Expected filename: snippet_metadata_{apishortname}_{apiversion}.json
+            snippet_metadata_path = str(pathlib.Path(
+                out_dir) / f"snippet_metadata_{api_schema.naming.name}_{api_schema.naming.version}.json").lower()
+            output_files[snippet_metadata_path] = CodeGeneratorResponse.File(
+                content=formatter.fix_whitespace(index.get_metadata_json()), name=snippet_metadata_path)
 
-            manifest_fname = os.path.join(out_dir, manifest_fname)
-            output_files[manifest_fname] = CodeGeneratorResponse.File(
-                content=manifest_doc.render(), name=manifest_fname
-            )
-
-        return output_files
+        return output_files, index
 
     def _render_template(
-            self, template_name: str, *, api_schema: api.API, opts: Options,
+            self, template_name: str, *, api_schema: api.API, opts: Options, snippet_index: snippet_index.SnippetIndex,
     ) -> Dict[str, CodeGeneratorResponse.File]:
         """Render the requested templates.
 
@@ -228,7 +239,11 @@ class Generator:
         answer: Dict[str, CodeGeneratorResponse.File] = OrderedDict()
         skip_subpackages = False
 
-        # Sanity check: Rendering per service and per proto would be a
+        # Very, very special case. This flag exists to gate this one file.
+        if not opts.metadata and template_name.endswith("gapic_metadata.json.j2"):
+            return answer
+
+        # Quick check: Rendering per service and per proto would be a
         # combinatorial explosion and is almost certainly not what anyone
         # ever wants. Error colorfully on it.
         if "%service" in template_name and "%proto" in template_name:
@@ -244,7 +259,7 @@ class Generator:
             for subpackage in api_schema.subpackages.values():
                 answer.update(
                     self._render_template(
-                        template_name, api_schema=subpackage, opts=opts
+                        template_name, api_schema=subpackage, opts=opts, snippet_index=snippet_index
                     )
                 )
                 skip_subpackages = True
@@ -261,7 +276,7 @@ class Generator:
 
                 answer.update(
                     self._get_file(
-                        template_name, api_schema=api_schema, proto=proto, opts=opts
+                        template_name, api_schema=api_schema, proto=proto, opts=opts, snippet_index=snippet_index
                     )
                 )
 
@@ -272,8 +287,15 @@ class Generator:
         if "%service" in template_name:
             for service in api_schema.services.values():
                 if (
-                        skip_subpackages
-                        and service.meta.address.subpackage != api_schema.subpackage_view
+                        (skip_subpackages
+                         and service.meta.address.subpackage != api_schema.subpackage_view)
+                        or
+                        ('transport' in template_name
+                         and not self._is_desired_transport(template_name, opts))
+                        or
+                        # TODO(yon-mg) - remove when rest async implementation resolved
+                        # temporarily stop async client gen while rest async is unkown
+                        ('async' in template_name and 'grpc' not in opts.transport)
                 ):
                     continue
 
@@ -283,6 +305,7 @@ class Generator:
                         api_schema=api_schema,
                         service=service,
                         opts=opts,
+                        snippet_index=snippet_index,
                     )
                 )
             return answer
@@ -290,16 +313,21 @@ class Generator:
         # This file is not iterating over anything else; return back
         # the one applicable file.
         answer.update(self._get_file(
-            template_name, api_schema=api_schema, opts=opts))
+            template_name, api_schema=api_schema, opts=opts, snippet_index=snippet_index))
         return answer
 
+    def _is_desired_transport(self, template_name: str, opts: Options) -> bool:
+        """Returns true if template name contains a desired transport"""
+        desired_transports = ['__init__', 'base'] + opts.transport
+        return any(transport in template_name for transport in desired_transports)
+
     def _get_file(
         self,
         template_name: str,
         *,
         opts: Options,
-        api_schema=api.API,
-        **context: Mapping,
+        api_schema: api.API,
+        **context,
     ):
         """Render a template to a protobuf plugin File object."""
         # Determine the target filename.
@@ -316,7 +344,7 @@ class Generator:
             name=fn,
         )
 
-        # Sanity check: Do not render empty files.
+        # Quick check: Do not render empty files.
         if utils.empty(cgr_file.content) and not fn.endswith(
             ("py.typed", "__init__.py")
         ):
@@ -354,7 +382,7 @@ class Generator:
         # Replace the %namespace variable.
         filename = filename.replace(
             "%namespace",
-            os.path.sep.join([i.lower() for i in api_schema.naming.namespace]),
+            os.path.sep.join(i.lower() for i in api_schema.naming.namespace),
         ).lstrip(os.path.sep)
 
         # Replace the %name, %version, and %sub variables.
