@@ -20,14 +20,17 @@ import keyword
 import os
 import re
 import time
+import yaml
 
 from gapic import utils
 
-from gapic.samplegen_utils import types
+from gapic.samplegen_utils import types, snippet_metadata_pb2  # type: ignore
+from gapic.samplegen_utils.utils import is_valid_sample_cfg
+from gapic.schema import api
 from gapic.schema import wrappers
 
-from collections import (defaultdict, namedtuple, ChainMap as chainmap)
-from typing import (ChainMap, Dict, FrozenSet, List, Mapping, Optional, Tuple)
+from collections import defaultdict, namedtuple, ChainMap as chainmap
+from typing import Any, ChainMap, Dict, FrozenSet, Generator, List, Mapping, Optional, Sequence, Tuple
 
 # There is no library stub file for this module, so ignore it.
 from google.api import resource_pb2  # type: ignore
@@ -71,6 +74,7 @@ class AttributeRequestSetup:
     that contains the value for the attribute.
 
     """
+
     value: str
     field: Optional[str] = None
     value_is_file: bool = False
@@ -98,6 +102,7 @@ class TransformedRequest:
 
     The Optional[single]/Optional[body] is workaround for not having tagged unions.
     """
+
     base: str
     single: Optional[AttributeRequestSetup]
     body: Optional[List[AttributeRequestSetup]]
@@ -105,11 +110,17 @@ class TransformedRequest:
 
     # Resource patterns look something like
     # kingdom/{kingdom}/phylum/{phylum}/class/{class}
-    RESOURCE_RE = re.compile(r"\{([^}/]+)\}")
+    RESOURCE_RE = wrappers.MessageType.PATH_ARG_RE
 
     @classmethod
-    def build(cls, request_type: wrappers.MessageType, api_schema, base: str,
-              attrs: List[AttributeRequestSetup], is_resource_request: bool):
+    def build(
+        cls,
+        request_type: wrappers.MessageType,
+        api_schema: api.API,
+        base: str,
+        attrs: List[AttributeRequestSetup],
+        is_resource_request: bool,
+    ):
         """Build a TransformedRequest based on parsed input.
 
         Acts as a factory to hide complicated logic for resource-based requests.
@@ -152,36 +163,48 @@ class TransformedRequest:
             #
             # It's a precondition that the base field is
             # a valid field of the request message type.
-            resource_typestr = (request_type.
-                                fields[base].
-                                options.
-                                Extensions[resource_pb2.resource_reference].
-                                type)
+            resource_reference = request_type.fields[base].options.Extensions[resource_pb2.resource_reference]
+            resource_typestr = resource_reference.type or resource_reference.child_type
 
-            resource_message_descriptor = next(
-                (msg.options.Extensions[resource_pb2.resource]
-                 for msg in api_schema.messages.values()
-                 if msg.options.Extensions[resource_pb2.resource].type == resource_typestr),
-                None
-            )
-            if not resource_message_descriptor:
+            resource_message = None
+            for service in api_schema.services.values():
+                resource_message = service.resource_messages_dict.get(
+                    resource_typestr)
+                if resource_message is not None:
+                    break
+
+            if resource_message is None:
                 raise types.NoSuchResource(
-                    f"No message exists for resource: {resource_typestr}")
+                    f"No message exists for resource: {resource_typestr}",
+                )
+            resource_message_descriptor = resource_message.options.Extensions[
+                resource_pb2.resource]
 
             # The field is only ever empty for singleton attributes.
             attr_names: List[str] = [a.field for a in attrs]  # type: ignore
 
             # A single resource may be found under multiple paths and have many patterns.
             # We want to find an _exact_ match, if one exists.
-            pattern = next((p
-                            for p in resource_message_descriptor.pattern
-                            if cls.RESOURCE_RE.findall(p) == attr_names), None)
+            pattern = next(
+                (
+                    p
+                    for p in resource_message_descriptor.pattern
+                    if cls.RESOURCE_RE.findall(p) == attr_names
+                ),
+                None,
+            )
             if not pattern:
                 attr_name_str = ", ".join(attr_names)
                 raise types.NoSuchResourcePattern(
-                    f"Resource {resource_typestr} has no pattern with params: {attr_name_str}")
+                    f"Resource {resource_typestr} has no pattern with params: {attr_name_str}"
+                )
+            # This re-writes
+            # patterns like: 'projects/{project}/metricDescriptors/{metric_descriptor=**}'
+            # to 'projects/{project}/metricDescriptors/{metric_descriptor}
+            # so it can be used in sample code as an f-string.
+            pattern = cls.RESOURCE_RE.sub(r"{\g<1>}", pattern)
 
-            return cls(base=base, body=attrs, single=None, pattern=pattern)
+            return cls(base=base, body=attrs, single=None, pattern=pattern,)
 
 
 @dataclasses.dataclass
@@ -222,19 +245,20 @@ class Validator:
     EXPRESSION_ATTR_RE = re.compile(
         r"""
         (?P<attr_name>\$?\w+)(?:\[(?P<index>\d+)\]|\{["'](?P<key>[^"']+)["']\})?$
-        """.strip())
+        """.strip()
+    )
 
     VALID_REQUEST_KWORDS = frozenset(
-        ("value", "field", "value_is_file", "input_parameter", "comment"))
+        ("value", "field", "value_is_file", "input_parameter", "comment")
+    )
 
-    # TODO(dovs): make the schema a required param.
-    def __init__(self, method: wrappers.Method, api_schema=None):
+    def __init__(self, method: wrappers.Method, api_schema: api.API):
         # The response ($resp) variable is special and guaranteed to exist.
         self.method = method
         self.request_type_ = method.input
         response_type = method.output
         if method.paged_result_field:
-            response_type = method.paged_result_field
+            response_type = method.paged_result_field.message
         elif method.lro:
             response_type = method.lro.response_type
 
@@ -258,21 +282,53 @@ class Validator:
         )
 
     @staticmethod
-    def preprocess_sample(sample, api_schema):
+    def preprocess_sample(sample, api_schema: api.API, rpc: wrappers.Method):
         """Modify a sample to set default or missing fields.
 
         Args:
            sample (Any): A definition for a single sample generated from parsed yaml.
            api_schema (api.API): The schema that defines the API to which the sample belongs.
+           rpc (wrappers.Method): The rpc method used in the sample.
         """
         sample["package_name"] = api_schema.naming.warehouse_package_name
-        sample.setdefault("response", [{"print": ["%s", "$resp"]}])
+        sample["module_name"] = api_schema.naming.versioned_module_name
+        sample["module_namespace"] = api_schema.naming.module_namespace
+
+        service = api_schema.services[sample["service"]]
+
+        # Assume the gRPC transport if the transport is not specified
+        transport = sample.setdefault("transport", api.TRANSPORT_GRPC)
+
+        is_async = transport == api.TRANSPORT_GRPC_ASYNC
+        sample["client_name"] = service.async_client_name if is_async else service.client_name
+
+        # the MessageType of the request object passed to the rpc e.g, `ListRequest`
+        sample["request_type"] = rpc.input
+
+        # We check if the request object is part of the service proto package.
+        # If not, it comes from a different module.
+        address = rpc.input.meta.address
+        if address.proto_package.startswith(address.api_naming.proto_package):
+            sample["request_module_name"] = sample["module_name"]
+        else:
+            sample["request_module_name"] = address.python_import.module
+
+        # If no request was specified in the config
+        # Add reasonable default values as placeholders
+        if "request" not in sample:
+            sample["request"] = generate_request_object(
+                api_schema, service, rpc.input)
+
+        # If no response was specified in the config
+        # Add reasonable defaults depending on the type of the sample
+        if not rpc.void:
+            sample.setdefault("response", [{"print": ["%s", "$resp"]}])
+        else:
+            sample.setdefault("response", [])
 
     @utils.cached_property
     def flattenable_fields(self) -> FrozenSet[str]:
-        return frozenset(
-            field.name for field in self.method.flattened_fields.values()
-        )
+        return frozenset(field.name for field in self.method.flattened_fields.values())
 
     def var_field(self, var_name: str) -> Optional[wrappers.Field]:
         return self.var_defs_.get(var_name)
@@ -299,7 +355,9 @@ class Validator:
             if not attr:
                 raise types.BadAttributeLookup(
                     "Method request type {} has no attribute: '{}'".format(
-                        self.request_type_, attr_name))
+                        self.request_type_, attr_name
+                    )
+                )
 
             if attr.message:
                 base = attr.message
@@ -309,20 +367,23 @@ class Validator:
                 witness = any(e.name == val for e in attr.enum.values)
                 if not witness:
                     raise types.InvalidEnumVariant(
-                        "Invalid variant for enum {}: '{}'".format(attr, val))
+                        "Invalid variant for enum {}: '{}'".format(attr, val)
+                    )
                 break
             elif attr.is_primitive:
                 # Only valid if this is the last attribute in the chain.
                 break
             else:
                 raise TypeError(
-                    f"Could not handle attribute '{attr_name}' of type: {attr.type}")
+                    f"Could not handle attribute '{attr_name}' of type: {attr.type}"
+                )
 
         if i != len(attr_chain) - 1:
             # We broke out of the loop after processing an enum or a primitive.
             extra_attrs = ".".join(attr_chain[i:])
             raise types.NonTerminalPrimitiveOrEnum(
-                f"Attempted to reference attributes of enum value or primitive type: '{extra_attrs}'")
+                f"Attempted to reference attributes of enum value or primitive type: '{extra_attrs}'"
+            )
 
         if len(attr_chain) > 1:
             request["field"] = ".".join(attr_chain[1:])
@@ -333,7 +394,9 @@ class Validator:
             if attr_chain[0] in base_param_to_attrs:
                 raise types.InvalidRequestSetup(
                     "Duplicated top level field in request block: '{}'".format(
-                        attr_chain[0]))
+                        attr_chain[0]
+                    )
+                )
             del request["field"]
 
         if isinstance(request["value"], str):
@@ -351,9 +414,9 @@ class Validator:
         # so disable it for the AttributeRequestSetup ctor call.
         return attr_chain[0], AttributeRequestSetup(**request)  # type: ignore
 
-    def validate_and_transform_request(self,
-                                       calling_form: types.CallingForm,
-                                       request: List[Mapping[str, str]]) -> FullRequest:
+    def validate_and_transform_request(
+        self, calling_form: types.CallingForm, request: List[Mapping[str, str]]
+    ) -> FullRequest:
         """Validates and transforms the "request" block from a sample config.
 
            In the initial request, each dict has a "field" key that maps to a dotted
@@ -416,9 +479,7 @@ class Validator:
 
         Raises:
             InvalidRequestSetup: If a dict in the request lacks a "field" key,
-                                 a "value" key, if there is an unexpected keyword,
-                                 or if more than one base parameter is given for
-                                 a client-side streaming calling form.
+                                 a "value" key or if there is an unexpected keyword.
             BadAttributeLookup: If a request field refers to a non-existent field
                                 in the request message type.
             ResourceRequestMismatch: If a request attempts to describe both
@@ -427,70 +488,76 @@ class Validator:
 
         """
         base_param_to_attrs: Dict[str,
-                                  RequestEntry] = defaultdict(RequestEntry)
+            RequestEntry] = defaultdict(RequestEntry)
         for r in request:
             r_dup = dict(r)
             val = r_dup.get("value")
             if not val:
                 raise types.InvalidRequestSetup(
-                    "Missing keyword in request entry: 'value'")
+                    "Missing keyword in request entry: 'value'"
+                )
 
             field = r_dup.get("field")
             if not field:
                 raise types.InvalidRequestSetup(
-                    "Missing keyword in request entry: 'field'")
+                    "Missing keyword in request entry: 'field'"
+                )
 
             spurious_kwords = set(r_dup.keys()) - self.VALID_REQUEST_KWORDS
             if spurious_kwords:
                 raise types.InvalidRequestSetup(
                     "Spurious keyword(s) in request entry: {}".format(
-                        ", ".join(f"'{kword}'" for kword in spurious_kwords)))
+                        ", ".join(f"'{kword}'" for kword in spurious_kwords)
+                    )
+                )
 
             input_parameter = r_dup.get("input_parameter")
             if input_parameter:
-                self._handle_lvalue(input_parameter, wrappers.Field(
-                    field_pb=descriptor_pb2.FieldDescriptorProto()))
+                self._handle_lvalue(
+                    input_parameter,
+                    wrappers.Field(
+                        field_pb=descriptor_pb2.FieldDescriptorProto()),
+                )
 
             # The percentage sign is used for setting up resource based requests
-            percent_idx = field.find('%')
+            percent_idx = field.find("%")
             if percent_idx == -1:
                 base_param, attr = self._normal_request_setup(
-                    base_param_to_attrs, val, r_dup, field)
+                    base_param_to_attrs, val, r_dup, field
+                )
 
                 request_entry = base_param_to_attrs.get(base_param)
                 if request_entry and request_entry.is_resource_request:
                     raise types.ResourceRequestMismatch(
-                        f"Request setup mismatch for base: {base_param}")
+                        f"Request setup mismatch for base: {base_param}"
+                    )
 
                 base_param_to_attrs[base_param].attrs.append(attr)
             else:
                 # It's a resource based request.
-                base_param, resource_attr = (field[:percent_idx],
-                                             field[percent_idx + 1:])
+                base_param, resource_attr = (
+                    field[:percent_idx],
+                    field[percent_idx + 1:],
+                )
                 request_entry = base_param_to_attrs.get(base_param)
                 if request_entry and not request_entry.is_resource_request:
                     raise types.ResourceRequestMismatch(
-                        f"Request setup mismatch for base: {base_param}")
+                        f"Request setup mismatch for base: {base_param}"
+                    )
 
                 if not self.request_type_.fields.get(base_param):
                     raise types.BadAttributeLookup(
                         "Method request type {} has no attribute: '{}'".format(
-                            self.request_type_, base_param))
+                            self.request_type_, base_param
+                        )
+                    )
 
                 r_dup["field"] = resource_attr
                 request_entry = base_param_to_attrs[base_param]
                 request_entry.is_resource_request = True
                 request_entry.attrs.append(
-                    AttributeRequestSetup(**r_dup))  # type: ignore
-
-        client_streaming_forms = {
-            types.CallingForm.RequestStreamingClient,
-            types.CallingForm.RequestStreamingBidi,
-        }
-
-        if len(base_param_to_attrs) > 1 and calling_form in client_streaming_forms:
-            raise types.InvalidRequestSetup(
-                "Too many base parameters for client side streaming form")
+                    AttributeRequestSetup(**r_dup)  # type: ignore
+                )
 
         # We can only flatten a collection of request parameters if they're a
         # subset of the flattened fields of the method.
@@ -502,11 +569,11 @@ class Validator:
                     self.api_schema_,
                     key,
                     val.attrs,
-                    val.is_resource_request
+                    val.is_resource_request,
                 )
                 for key, val in base_param_to_attrs.items()
             ],
-            flattenable=flattenable
+            flattenable=False,
         )
 
     def validate_response(self, response):
@@ -535,7 +602,8 @@ class Validator:
             validater = self.STATEMENT_DISPATCH_TABLE.get(keyword)
             if not validater:
                 raise types.InvalidStatement(
-                    "Invalid statement keyword: {}".format(keyword))
+                    "Invalid statement keyword: {}".format(keyword)
+                )
 
             validater(self, body)
 
@@ -558,34 +626,45 @@ class Validator:
         Returns:
             wrappers.Field: The final field in the chain.
         """
+
         def validate_recursively(expression, scope, depth=0):
             first_dot = expression.find(".")
             base = expression[:first_dot] if first_dot > 0 else expression
             match = self.EXPRESSION_ATTR_RE.match(base)
             if not match:
                 raise types.BadAttributeLookup(
-                    f"Badly formed attribute expression: {expression}")
+                    f"Badly formed attribute expression: {expression}"
+                )
 
-            name, idxed, mapped = (match.groupdict()["attr_name"],
-                                   bool(match.groupdict()["index"]),
-                                   bool(match.groupdict()["key"]))
+            name, idxed, mapped = (
+                match.groupdict()["attr_name"],
+                bool(match.groupdict()["index"]),
+                bool(match.groupdict()["key"]),
+            )
             field = scope.get(name)
+
             if not field:
-                exception_class = (types.BadAttributeLookup if depth else
-                                   types.UndefinedVariableReference)
+                exception_class = (
+                    types.BadAttributeLookup
+                    if depth
+                    else types.UndefinedVariableReference
+                )
                 raise exception_class(f"No such variable or attribute: {name}")
 
             # Invalid input
             if (idxed or mapped) and not field.repeated:
                 raise types.BadAttributeLookup(
-                    f"Collection lookup on non-repeated field: {base}")
+                    f"Collection lookup on non-repeated field: {base}"
+                )
 
             # Can only ignore indexing or mapping in an indexed (or mapped) field
             # if it is the terminal point in the expression.
             if field.repeated and not (idxed or mapped) and first_dot != -1:
                 raise types.BadAttributeLookup(
-                    ("Accessing attribute on a non-terminal collection without"
-                     f"indexing into the collection: {base}")
+                    (
+                        "Accessing attribute on a non-terminal collection without"
+                        f"indexing into the collection: {base}"
+                    )
                 )
 
             message = field.message
@@ -601,12 +680,14 @@ class Validator:
                 value_field = message.fields.get("value")
                 if not value_field:
                     raise types.BadAttributeLookup(
-                        f"Mapped attribute has no value field: {base}")
+                        f"Mapped attribute has no value field: {base}"
+                    )
 
                 value_message = value_field.message
                 if not value_message:
                     raise types.BadAttributeLookup(
-                        f"Mapped value field is not a message: {base}")
+                        f"Mapped value field is not a message: {base}"
+                    )
 
                 if first_dot != -1:
                     scope = value_message.fields
@@ -618,11 +699,10 @@ class Validator:
             # Enums and primitives are only allowed at the tail of an expression.
             if not message:
                 raise types.BadAttributeLookup(
-                    f"Non-terminal attribute is not a message: {base}")
+                    f"Non-terminal attribute is not a message: {base}"
+                )
 
-            return validate_recursively(expression[first_dot + 1:],
-                                        scope,
-                                        depth + 1)
+            return validate_recursively(expression[first_dot + 1:], scope, depth + 1)
 
         return validate_recursively(exp, self.var_defs_)
 
@@ -664,9 +744,7 @@ class Validator:
         if num_prints != len(body) - 1:
             raise types.MismatchedFormatSpecifier(
                 "Expected {} expresssions in format string '{}' but found {}".format(
-                    num_prints,
-                    fmt_str,
-                    len(body) - 1
+                    num_prints, fmt_str, len(body) - 1
                 )
             )
 
@@ -714,14 +792,16 @@ class Validator:
         fname_fmt = body.get("filename")
         if not fname_fmt:
             raise types.InvalidStatement(
-                "Missing key in 'write_file' statement: 'filename'")
+                "Missing key in 'write_file' statement: 'filename'"
+            )
 
         self._validate_format(fname_fmt)
 
         contents_var = body.get("contents")
         if not contents_var:
             raise types.InvalidStatement(
-                "Missing key in 'write_file' statement: 'contents'")
+                "Missing key in 'write_file' statement: 'contents'"
+            )
 
         self.validate_expression(contents_var)
 
@@ -775,13 +855,14 @@ class Validator:
             # TODO: resolve the implicit $resp dilemma
             # if collection_name.startswith("."):
             #     collection_name = "$resp" + collection_name
-            collection_field = self.validate_expression(
-                loop[self.COLL_KWORD])
+            collection_field = self.validate_expression(loop[self.COLL_KWORD])
 
             if not collection_field.repeated:
                 raise types.BadLoop(
                     "Tried to use a non-repeated field as a collection: {}".format(
-                        tokens[-1]))
+                        tokens[-1]
+                    )
+                )
 
             var = loop[self.VAR_KWORD]
             # The collection_field is repeated,
@@ -792,8 +873,8 @@ class Validator:
                     field_pb=collection_field.field_pb,
                     message=collection_field.message,
                     enum=collection_field.enum,
-                    meta=collection_field.meta
-                )
+                    meta=collection_field.meta,
+                ),
             )
 
         elif map_args <= segments:
@@ -817,7 +898,8 @@ class Validator:
 
             if not (key or val):
                 raise types.BadLoop(
-                    "Need at least one of 'key' or 'value' in a map loop")
+                    "Need at least one of 'key' or 'value' in a map loop"
+                )
 
         else:
             raise types.BadLoop("Unexpected loop form: {}".format(segments))
@@ -838,22 +920,201 @@ class Validator:
     }
 
 
-def generate_sample(
-        sample,
-        api_schema,
-        sample_template: jinja2.Template
-) -> str:
+def parse_handwritten_specs(sample_configs: Sequence[str]) -> Generator[Dict[str, Any], None, None]:
+    """Parse a handwritten sample spec"""
+
+    for config_fpath in sample_configs:
+        with open(config_fpath) as f:
+            configs = yaml.safe_load_all(f.read())
+
+            for cfg in configs:
+                valid = is_valid_sample_cfg(cfg)
+                if not valid:
+                    raise types.InvalidConfig(
+                        "Sample config in '{}' is invalid\n\n{}".format(config_fpath, cfg), valid)
+                for spec in cfg.get("samples", []):
+                    yield spec
+
+
+def generate_request_object(api_schema: api.API, service: wrappers.Service, message: wrappers.MessageType, field_name_prefix: str = ""):
+    """Generate dummy input for a given message.
+
+    Args:
+        api_schema (api.API): The schema that defines the API.
+        service (wrappers.Service): The service object the message belongs to.
+        message (wrappers.MessageType): The message to generate a request object for.
+        field_name_prefix (str): A prefix to attach to the field name in the request.
+
+    Returns:
+        List[Dict[str, Any]]: A list of dicts that can be turned into TransformedRequests.
+    """
+    request: List[Dict[str, Any]] = []
+
+    request_fields: List[wrappers.Field] = []
+
+    # There is no standard syntax to mark a oneof as "required" in protos.
+    # Assume every oneof is required and pick the first option
+    # in each oneof.
+    selected_oneofs: List[wrappers.Field] = [oneof_fields[0]
+        for oneof_fields in message.oneof_fields().values()]
+
+    # Don't add required fields if they're also marked as oneof
+    required_fields = [
+        field for field in message.required_fields if not field.oneof]
+    request_fields = selected_oneofs + required_fields
+
+    for field in request_fields:
+        # TransformedRequest expects nested fields to be referenced like
+        # `destination.input_config.name`
+        field_name = ".".join([field_name_prefix, field.name]).lstrip('.')
+
+        # TODO(busunkim): Properly handle map fields
+        if field.is_primitive:
+            request.append(
+                {"field": field_name, "value": field.mock_value_original_type})
+        elif field.enum:
+            # Choose the last enum value in the list since index 0 is often "unspecified"
+            request.append(
+                {"field": field_name, "value": field.enum.values[-1].name})
+        else:
+            # This is a message type, recurse
+            # TODO(busunkim):  Some real world APIs have
+            # request objects are recursive.
+            # Reference `Field.mock_value` to ensure
+            # this always terminates.
+            request += generate_request_object(
+                api_schema, service, field.type,
+                field_name_prefix=field_name,
+            )
+
+    return request
+
+
+def generate_sample_specs(api_schema: api.API, *, opts) -> Generator[Dict[str, Any], None, None]:
+    """Given an API, generate basic sample specs for each method.
+
+    Args:
+        api_schema (api.API): The schema that defines the API.
+
+    Yields:
+        Dict[str, Any]: A sample spec.
+    """
+
+    gapic_metadata = api_schema.gapic_metadata(opts)
+
+    for service_name, service in gapic_metadata.services.items():
+        api_short_name = api_schema.services[f"{api_schema.naming.proto_package}.{service_name}"].shortname
+        api_version = api_schema.naming.version
+        for transport, client in service.clients.items():
+            transport_type = "async" if transport == api.TRANSPORT_GRPC_ASYNC else "sync"
+            for rpc_name, method_list in client.rpcs.items():
+                # Region Tag Format:
+                # [{START|END} ${apishortname}_${apiVersion}_generated_${serviceName}_${rpcName}_{sync|async}]
+                region_tag = f"{api_short_name}_{api_version}_generated_{service_name}_{rpc_name}_{transport_type}"
+                spec = {
+                    "rpc": rpc_name,
+                    "transport": transport,
+                    # `request` and `response` is populated in `preprocess_sample`
+                    "service": f"{api_schema.naming.proto_package}.{service_name}",
+                    "region_tag": region_tag,
+                    "description": f"Snippet for {utils.to_snake_case(rpc_name)}"
+                }
+
+                yield spec
+
+
+def _fill_sample_metadata(sample: dict, api_schema: api.API):
+    """Returns snippet metadata for the sample."""
+
+    # Snippet Metadata can't be fully filled out in any one function
+    # In this function we add information from
+    # the API schema and sample dictionary.
+    # See `snippet_metadata.proto` for documentation on the fields
+
+    service = api_schema.services[sample["service"]]
+    method = service.methods[sample["rpc"]]
+    async_ = sample["transport"] == api.TRANSPORT_GRPC_ASYNC
+
+    snippet_metadata = snippet_metadata_pb2.Snippet()  # type: ignore
+    snippet_metadata.region_tag = sample["region_tag"]
+    snippet_metadata.description = f"Sample for {sample['rpc']}"
+    snippet_metadata.language = snippet_metadata_pb2.Language.PYTHON  # type: ignore
+    snippet_metadata.canonical = True
+    snippet_metadata.origin = snippet_metadata_pb2.Snippet.Origin.API_DEFINITION  # type: ignore
+
+    # Service Client
+    snippet_metadata.client_method.client.short_name = service.async_client_name if async_ else service.client_name
+    snippet_metadata.client_method.client.full_name = f"{'.'.join(sample['module_namespace'])}.{sample['module_name']}.{snippet_metadata.client_method.client.short_name}"
+
+    # Service
+    snippet_metadata.client_method.method.service.short_name = service.name
+    snippet_metadata.client_method.method.service.full_name = f"{api_schema.naming.proto_package}.{service.name}"
+
+    # RPC
+    snippet_metadata.client_method.method.short_name = method.name
+    snippet_metadata.client_method.method.full_name = f"{api_schema.naming.proto_package}.{service.name}.{method.name}"
+
+    # Client Method
+    setattr(snippet_metadata.client_method, "async", async_)
+    snippet_metadata.client_method.short_name = utils.to_snake_case(
+        method.name)
+    snippet_metadata.client_method.full_name = f"{snippet_metadata.client_method.client.full_name}.{snippet_metadata.client_method.short_name}"
+
+    if not method.void:
+        snippet_metadata.client_method.result_type = method.client_output_async.ident.sphinx if async_ else method.client_output.ident.sphinx
+        if method.server_streaming:
+            snippet_metadata.client_method.result_type = f"Iterable[{snippet_metadata.client_method.result_type }]"
+
+    # Client Method Parameters
+    parameters = snippet_metadata.client_method.parameters
+    if not method.client_streaming:
+        parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+            type=method.input.ident.sphinx, name="request"))
+        for field in method.flattened_fields.values():
+            parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+                type=field.ident.sphinx, name=field.name))
+    else:
+        parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+            type=f"Iterator[{method.input.ident.sphinx}]", name="requests"))
+
+    parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+        name="retry", type="google.api_core.retry.Retry"))
+    parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+        name="timeout", type="float"))
+    parameters.append(snippet_metadata_pb2.ClientMethod.Parameter(  # type: ignore
+        name="metadata", type="Sequence[Tuple[str, str]"))
+
+    return snippet_metadata
+
+
+def _get_sample_imports(sample: Dict, rpc: wrappers.Method) -> List[str]:
+    """Returns sorted sample import statements."""
+    module_namespace = ".".join(sample["module_namespace"])
+    module_name = sample["module_name"]
+    module_import = f"from {module_namespace} import {module_name}"
+
+    address = rpc.input.meta.address
+    # This checks if the request message is part of the service proto package.
+    # If not, we should try to include a separate import statement.
+    if address.proto_package.startswith(address.api_naming.proto_package):
+        return [module_import]
+    else:
+        request_import = str(address.python_import)
+        return sorted([module_import, request_import])
+
+
+def generate_sample(sample, api_schema, sample_template: jinja2.Template) -> Tuple[str, Any]:
     """Generate a standalone, runnable sample.
 
     Writing the rendered output is left for the caller.
 
     Args:
-        sample (Any): A definition for a single sample generated from parsed yaml.
+        sample (Any): A definition for a single sample.
         api_schema (api.API): The schema that defines the API to which the sample belongs.
         sample_template (jinja2.Template): The template representing a generic sample.
 
     Returns:
-        str: The rendered sample.
+        Tuple(str, snippet_metadata_pb2.Snippet): The rendered sample.
     """
     service_name = sample["service"]
     service = api_schema.services.get(service_name)
@@ -870,22 +1131,26 @@ def generate_sample(
 
     calling_form = types.CallingForm.method_default(rpc)
 
-    v = Validator(rpc)
-    # Tweak some small aspects of the sample to set sane defaults for optional
+    v = Validator(rpc, api_schema)
+    # Tweak some small aspects of the sample to set defaults for optional
     # fields, add fields that are required for the template, and so forth.
-    v.preprocess_sample(sample, api_schema)
-    sample["request"] = v.validate_and_transform_request(calling_form,
-                                                         sample["request"])
+    v.preprocess_sample(sample, api_schema, rpc)
+    sample["request"] = v.validate_and_transform_request(
+        calling_form, sample["request"]
+    )
+
     v.validate_response(sample["response"])
 
+    snippet_metadata = _fill_sample_metadata(sample, api_schema)
+
+    # The sample must be preprocessed before calling _get_sample_imports.
+    imports = _get_sample_imports(sample, rpc)
+
     return sample_template.render(
         sample=sample,
-        imports=[
-            "from google import auth",
-            "from google.auth import credentials",
-        ],
+        imports=imports,
         calling_form=calling_form,
         calling_form_enum=types.CallingForm,
-        api=api_schema,
-        service=service,
-    )
+        trim_blocks=True,
+        lstrip_blocks=True,
+    ), snippet_metadata
