@@ -43,6 +43,7 @@ import grpc  # type: ignore
 from google.protobuf.descriptor_pb2 import MethodDescriptorProto
 from google.api import annotations_pb2  # type: ignore
 from gapic.schema import metadata
+from gapic.schema import mixins
 from gapic.schema import wrappers
 from gapic.schema import naming as api_naming
 from gapic.utils import cached_property
@@ -515,6 +516,16 @@ class API:
 
         return op_serv
 
+    @cached_property
+    def mixin_api_signatures(self):
+        """Compile useful info about MixIn API signatures.
+
+        Returns:
+            Mapping[str, wrappers.MixinMethod]: Useful info
+                about MixIn methods present for the main API.
+        """
+        return {name: mixins.MIXINS_MAP[name] for name in self.mixin_api_methods}
+
     @cached_property
     def mixin_api_methods(self) -> Dict[str, MethodDescriptorProto]:
         methods: Dict[str, MethodDescriptorProto] = {}
@@ -529,6 +540,20 @@ class API:
                 self._get_methods_from_service(operations_pb2)}
         return methods
 
+    @cached_property
+    def mixin_http_options(self):
+        """Gather HTTP options for the MixIn methods."""
+        api_methods = self.mixin_api_methods
+        res = {}
+        for s in api_methods:
+            m = api_methods[s]
+            http = m.options.Extensions[annotations_pb2.http]
+            http_options = [http] + list(http.additional_bindings)
+            opt_gen = (wrappers.MixinHttpRule.try_parse_http_rule(http_rule)
+                   for http_rule in http_options)
+            res[s] = [rule for rule in opt_gen if rule]
+        return res
+
     @cached_property
     def has_location_mixin(self) -> bool:
         return len(list(filter(lambda api: api.name == "google.cloud.location.Locations", self.service_yaml_config.apis))) > 0
