@@ -14,6 +14,9 @@
 
 from gapic.utils.cache import cached_property
 from gapic.utils.case import to_snake_case
+from gapic.utils.case import to_camel_case
+from gapic.utils.checks import is_msg_field_pb
+from gapic.utils.checks import is_str_field_pb
 from gapic.utils.code import empty
 from gapic.utils.code import nth
 from gapic.utils.code import partition
@@ -25,12 +28,16 @@ from gapic.utils.lines import wrap
 from gapic.utils.options import Options
 from gapic.utils.reserved_names import RESERVED_NAMES
 from gapic.utils.rst import rst
+from gapic.utils.uri_conv import convert_uri_fieldnames
 
 
 __all__ = (
     'cached_property',
+    'convert_uri_fieldnames',
     'doc',
     'empty',
+    'is_msg_field_pb',
+    'is_str_field_pb',
     'nth',
     'Options',
     'partition',
@@ -38,6 +45,7 @@ __all__ = (
     'rst',
     'sort_lines',
     'to_snake_case',
+    'to_camel_case',
     'to_valid_filename',
     'to_valid_module_name',
     'wrap',
