@@ -210,6 +210,13 @@ class IAMCredentialsAsyncClient:
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             async def sample_generate_access_token():
@@ -219,7 +226,7 @@ class IAMCredentialsAsyncClient:
                 # Initialize request argument(s)
                 request = credentials_v1.GenerateAccessTokenRequest(
                     name="name_value",
-                    scope=['scope_value_1', 'scope_value_2'],
+                    scope=['scope_value1', 'scope_value2'],
                 )
 
                 # Make the request
@@ -363,6 +370,13 @@ initial=0.1,maximum=60.0,multiplier=1.3,                predicate=retries.if_exc
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             async def sample_generate_id_token():
@@ -509,6 +523,13 @@ initial=0.1,maximum=60.0,multiplier=1.3,                predicate=retries.if_exc
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             async def sample_sign_blob():
@@ -642,6 +663,13 @@ initial=0.1,maximum=60.0,multiplier=1.3,                predicate=retries.if_exc
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             async def sample_sign_jwt():
